#pragma once

/* Genode includes */
#include <util/misc_math.h>
#include <util/string.h>
#include <base/lock.h>
#include <base/rpc_client.h>
#include <base/attached_dataspace.h>
#include <sign_session/sign_session.h>

namespace Sign
{
	using Genode::size_t;
	using Genode::uint8_t;

	class Session_client;
}

class Sign::Session_client : public Genode::Rpc_client<Session>
{
	private:

		Genode::Lock _lock { };

		/**
		 * Shared-memory buffer used for carrying the payload
		 * of read/write operations
		 */
		Genode::Attached_dataspace _io_buffer;

	public:

		Session_client(Genode::Region_map &local_rm, Genode::Capability<Session> cap)
		:
			Genode::Rpc_client<Session>(cap),
			_io_buffer(local_rm, call<Rpc_dataspace>())
		{ }

		Genode::size_t sign(void* data_buffer, Genode::size_t data_bytes, Genode::size_t buffer_size) override
		{
			Genode::Lock::Guard _guard(_lock);

			if (data_bytes > _io_buffer.size()) throw Buffer_too_small();

			Genode::memcpy(_io_buffer.local_addr<char>(), data_buffer, data_bytes);

			Genode::size_t reply_bytes = call<Rpc_sign>(data_bytes);

			if (reply_bytes > _io_buffer.size()) throw Buffer_too_small();
			if (reply_bytes > buffer_size) throw Buffer_too_small();

			Genode::memcpy(data_buffer, _io_buffer.local_addr<char>(), reply_bytes);

			return reply_bytes;
		}
//        size_t do_sign(const uint8_t* data_buffer, const size_t data_bytes,
//                       uint8_t* signature_buffer, size_t signature_buffer_size)
//        {
//            if ((data_buffer == nullptr) || (data_bytes == 0) ||
//                (signature_buffer == nullptr) || (signature_buffer_size == 0)) {
//                return 0;
//            }
//
//            sign
//        }

};

